﻿using System.Linq;
using System.Data.Objects;
using PhotoTagger.Model;

namespace PhotoTagger.Repository
{

    /// <summary>
    /// Extension methods for working with entities in the repository.
    /// </summary>
    internal static class RepositoryExtensions
    {

        /// <summary>
        /// Start tracking changes for all loaded objects in the current ObjectContext instance.
        /// </summary>
        public static void StartTrackingAll(this ObjectContext context)
        {
            var entities = context
                .ObjectStateManager
                .GetObjectStateEntries(System.Data.EntityState.Unchanged)
                .Select(x => x.Entity as IObjectWithChangeTracker)
                .Where(x => x != null);
            foreach (var x in entities)
            {
                x.StartTracking();
            }
        }

        /// <summary>
        /// Refresh the objects by reloading them from the store. Applies to all loaded objects in the current ObjectContext instance.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1804:RemoveUnusedLocals", MessageId = "deleted")]
        public static void RefreshAll(this ObjectContext context, RefreshMode refreshMode)
        {
            var entities = context
                .ObjectStateManager
                .GetObjectStateEntries(System.Data.EntityState.Unchanged)
                .Select(x => x.Entity as IObjectWithChangeTracker)
                .Where(x => x != null)
                .ToList();

            context.Refresh(refreshMode, entities);

            var deleted = context
                .ObjectStateManager
                .GetObjectStateEntries(System.Data.EntityState.Deleted)
                .ToList();
        }


        /// <summary>
        /// Mark the change tracker's state as "Unchanged" for all "Added" entities.
        /// This prevents the system from attempting to re-insert, which will cause a duplicate key exception.
        /// </summary>
        public static void MarkAllAddedAsUnmodified(this ObjectContext context)
        {
            var entities = context
                .ObjectStateManager
                .GetObjectStateEntries(System.Data.EntityState.Unchanged)
                .Select(x => x.Entity as IObjectWithChangeTracker)
                .Where(x => x != null && x.ChangeTracker.State == ObjectState.Added)
                .ToList();

            foreach (var e in entities)
            {
                e.MarkAsUnchanged();
            }

            entities = context
                .ObjectStateManager
                .GetObjectStateEntries(System.Data.EntityState.Added)
                .Select(x => x.Entity as IObjectWithChangeTracker)
                .Where(x => x != null && x.ChangeTracker.State == ObjectState.Added)
                .ToList();

            foreach (var e in entities)
            {
                e.MarkAsUnchanged();
            }
        }

        /// <summary>
        /// This returns whether the Object entity has any changes or not
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Maintainability", "CA1502:AvoidExcessiveComplexity")]
        public static bool HasChanges(this ObjectContext context)
        {

            var entities = context
                .ObjectStateManager
                .GetObjectStateEntries(System.Data.EntityState.Unchanged)
                .Select(x => x.Entity as IObjectWithChangeTracker)
                .Where(x => x != null && (x.ChangeTracker.State == ObjectState.Added || x.ChangeTracker.State == ObjectState.Modified || x.ChangeTracker.State == ObjectState.Deleted))
                .ToList();

            if (entities.Count > 0)
                return true;

            entities = context
                .ObjectStateManager
                .GetObjectStateEntries(System.Data.EntityState.Added)
                .Select(x => x.Entity as IObjectWithChangeTracker)
                .Where(x => x != null && (x.ChangeTracker.State == ObjectState.Added || x.ChangeTracker.State == ObjectState.Modified || x.ChangeTracker.State == ObjectState.Deleted))
                .ToList();

            if (entities.Count > 0)
                return true;

            entities = context
               .ObjectStateManager
               .GetObjectStateEntries(System.Data.EntityState.Modified)
               .Select(x => x.Entity as IObjectWithChangeTracker)
               .Where(x => x != null && (x.ChangeTracker.State == ObjectState.Added || x.ChangeTracker.State == ObjectState.Modified || x.ChangeTracker.State == ObjectState.Deleted))
               .ToList();

            if (entities.Count > 0)
                return true;

            entities = context
              .ObjectStateManager
              .GetObjectStateEntries(System.Data.EntityState.Deleted)
              .Select(x => x.Entity as IObjectWithChangeTracker)
              .Where(x => x != null && (x.ChangeTracker.State == ObjectState.Added || x.ChangeTracker.State == ObjectState.Modified || x.ChangeTracker.State == ObjectState.Deleted))
              .ToList();

            if (entities.Count > 0)
                return true;

            return false;
        }

        ///// <summary>
        ///// Accept changes for all loaded objects in the current ObjectContext instance.
        ///// Call this after saving the entities.
        ///// </summary>
        public static void AcceptChangesAll(this ObjectContext context)
        {


            var entities = context
                .ObjectStateManager
                .GetObjectStateEntries(System.Data.EntityState.Unchanged)
                .Select(x => x.Entity as IObjectWithChangeTracker)
                .Where(x => x != null);
            foreach (var x in entities)
            {
                x.AcceptChanges();
            }
        }

    }
}
