﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PhotoTagger.Model;

namespace PhotoTagger.Repository.SQL
{
    public class CategoryRepositorySQL : ICategoryRepository
    {
        
        #region ICategoryRepository Members

        public IEnumerable<Category> GetAllCategories()
        {
            using (new DelayedInitializationScope())
            {
                IEnumerable<Category> categories = null;

                using (var context = new PhotoModelContainer())
                {
                    categories = context
                        .Categories
                        .ToList();

                    // Start change tracking for all objects loaded in the current context.
                    context.StartTrackingAll();
                }

                return categories;
            }
        }

        public void Save(Category category)
        {
            if (category == null)
                throw new ArgumentNullException("category");

            using (var context = new PhotoModelContainer())
            {

                // This reviews the changes in the STE's change tracker and applies them to the context.
                // Updating the context lets the context know what fields need to be saved to the DB. Only changed fields are saved.
                context.Categories.ApplyChanges(category);

                // Save the changes to the DB. Only updated fields on affected tables are persisted.
                // Default behavior is to "AcceptAllChanges" after saving to the DB.
                // Default behavior also turns off change tracking.
                context.SaveChanges();

                // Mark added entities as "Unchanged" so that they are updated, not re-inserted.
                context.MarkAllAddedAsUnmodified();

                // On save, some columns may change in the DB. This includes Timestamps, columns with defaults,
                // and anything with a trigger.
                // The following call will re-load all values from the DB into the existing objects.
                // Changed fields will run through the property setters, and thereby update any viewmodels or 
                // UI elements that subscribe to the INotifyPropertyChange interface.
                using (new DelayedInitializationScope())
                {
                    context.RefreshAll(System.Data.Objects.RefreshMode.ClientWins);
                }

                context.AcceptChangesAll();

                // Since change tracking was turned off, we need to turn it back on.
                context.StartTrackingAll();
            }
        }

        public void Save(IEnumerable<Category> categories)
        {
            if (categories == null)
                throw new ArgumentNullException("categories");

            using (var context = new PhotoModelContainer())
            {

                // This reviews the changes in the STE's change tracker and applies them to the context.
                // Updating the context lets the context know what fields need to be saved to the DB. Only changed fields are saved.
                foreach (var category in categories)
                {
                    context.Categories.ApplyChanges(category);
                }

                // Save the changes to the DB. Only updated fields on affected tables are persisted.
                // Default behavior is to "AcceptAllChanges" after saving to the DB.
                // Default behavior also turns off change tracking.
                context.SaveChanges();

                // Mark added entities as "Unchanged" so that they are updated, not re-inserted.
                context.MarkAllAddedAsUnmodified();

                // On save, some columns may change in the DB. This includes Timestamps, columns with defaults,
                // and anything with a trigger.
                // The following call will re-load all values from the DB into the existing objects.
                // Changed fields will run through the property setters, and thereby update any viewmodels or 
                // UI elements that subscribe to the INotifyPropertyChange interface.
                using (new DelayedInitializationScope())
                {
                    context.RefreshAll(System.Data.Objects.RefreshMode.ClientWins);
                }

                context.AcceptChangesAll();

                // Since change tracking was turned off, we need to turn it back on.
                context.StartTrackingAll();
            }
        }

        #endregion
    }
}
