﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PhotoTagger.Model;
using System.Data.Objects;

namespace PhotoTagger.Repository.SQL
{
    public class PhotoRepositorySQL : IPhotoRepository
    {
        
        #region IPhotoRepository Members

        public Catalog GetCatalog(string catalogName)
        {
            //********************************************************************
            // Delayed Initialization Scope prevents the entities from firing INPC
            // while the entities are being loaded from the DB.
            //********************************************************************
            using (new DelayedInitializationScope())
            {
                Catalog catalog = null;

                using (var context = new PhotoModelContainer())
                {
                    catalog = context
                                .Catalogs
                                .Include("Photos")  // This tells EF to get any Photo records that are related to the Catalog item.
                                .Include("Photos.Categories")   // This tells EF to get the Categories related to the Photo that is related to the Catalog.
                                .Where(c => c.Name == catalogName)
                                .FirstOrDefault();


                    //********************************************************************
                    // This is an alternate way of doing the ".Include()" logic.
                    //  - .Include() issues a single query to get all records.
                    //  - .LoadProperty() issues a separate query to get the property.
                    // NOTE:
                    // Doing everything in a single query may sound better, but the 
                    // complexity of the query and the size of the resulting SQL statement
                    // can quickly outweigh any benefit. Often times, LoadProperty can 
                    // actually be the faster way to go. Use performance testing to determine
                    // the best approach.
                    //********************************************************************
                    //context.LoadProperty(catalog, c => c.Photos, MergeOption.PreserveChanges);
                    //if (null != catalog.Photos)
                    //{
                    //    foreach (var photo in catalog.Photos)
                    //    {
                    //        context.LoadProperty(photo, p => p.Categories, MergeOption.PreserveChanges);
                    //    }
                    //}


                    //********************************************************************
                    // Start change tracking for all objects loaded in the current context.
                    //********************************************************************
                    context.StartTrackingAll();
                }

                return catalog;
            }
        }

        public void Save(Catalog catalog)
        {
            if (catalog == null)
                throw new ArgumentNullException("catalog");

            using (var context = new PhotoModelContainer())
            {

                //********************************************************************
                // This reviews the changes in the STE's change tracker and applies 
                // them to the context.
                // 
                // Updating the context lets the context know what fields need to be 
                // saved to the DB. Only changed fields are saved.
                //********************************************************************
                context.Catalogs.ApplyChanges(catalog);

                //********************************************************************
                // Save the changes to the DB. Only updated fields on affected tables 
                // are persisted.
                // 
                // Default behavior is to "AcceptAllChanges" after saving to the DB.
                // Default behavior also turns off change tracking.
                //********************************************************************
                context.SaveChanges();

                //********************************************************************
                // Mark added entities as "Unchanged" so that they are updated, 
                // not re-inserted on future saves.
                //********************************************************************
                context.MarkAllAddedAsUnmodified();

                //********************************************************************
                // On save, some columns may change in the DB. This includes Timestamps, 
                // columns with defaults, and anything with a trigger.
                // 
                // The following call will re-load all values from the DB into the 
                // existing objects.
                // 
                // Changed fields will run through the property setters, and thereby 
                // update any viewmodels or UI elements that subscribe to the 
                // INotifyPropertyChange interface.
                //********************************************************************
                using (new DelayedInitializationScope())
                {
                    context.RefreshAll(System.Data.Objects.RefreshMode.ClientWins);
                }

                //********************************************************************
                // Discard any unsaved changes. Mark objects as unmodified.
                //********************************************************************
                context.AcceptChangesAll();

                //********************************************************************
                // Since change tracking was turned off, we need to turn it back on.
                //********************************************************************
                context.StartTrackingAll();
            }
        }


        #endregion
    }
}
