﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using ContinuousLinq;
using ContinuousLinq.Aggregates;
using ContinuousLinq.Reactive;
using Microsoft.Win32;
using MonitoredUndo;
using PhotoTagger.Model;
using PhotoTagger.Repository;
using System.Collections.Specialized;

namespace PhotoTagger.ViewModels
{
    public class MainViewModel : ReactiveObject, INotifyPropertyChanged, ISupportsUndo
    {
        
        #region Member Variables

        private Catalog _Catalog;
        private Photo _SelectedPhoto;
        private bool _OnlyShowFavorites;
        private bool? _UseSqlRepository;

        #endregion
        #region Constructors

        /// <summary>
        /// Static constructor. Registers the Reactive dependencies.
        /// </summary>
        static MainViewModel()
        {
            var dependsOn = Register<MainViewModel>();

            dependsOn.Call(me => me.OnPropertyChanged("AppTitle"))
                .OnChanged(me => me.Catalog.Name)
                .OnChanged(me => me.TotalPhotos);
        }

        public MainViewModel()
        {
            try
            {
                // On load, try to get the existing catalog.
                this.GetAllPhotos();

                if (null != Catalog && Catalog.Photos.Count > 0)
                    this.SelectedPhoto = Catalog.Photos.FirstOrDefault();

                UndoService.Current[Catalog].Clear();
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Unable to load Main catalog. " + ex.ToString());
            }
        }

        #endregion
        #region Properties

        public Catalog Catalog
        {
            get
            {
                return _Catalog;
            }
            private set
            {
                _Catalog = value;

                // Reset these since the catalog has changed.
                _TotalPhotos = null;
                _FavoritePhotos = null;

                OnPropertyChanged("Catalog");
                OnPropertyChanged("Photos");
                OnPropertyChanged("TotalPhotos");
                OnPropertyChanged("FavoritePhotos");
            }
        }
        
        public IList<Photo> Photos
        {
            get
            {
                if (null != _Catalog)
                    return _Catalog.Photos
                                   .AsReadOnly()
                                   .Where(
                                        p => p.IsFavorite || !this.OnlyShowFavorites
                                   );
                else
                    return null;
            }
        }


        public Photo SelectedPhoto
        {
            get { return _SelectedPhoto; }
            set
            {
                if (value == _SelectedPhoto)
                    return;

                // This will make the selection change "undoable"
                UndoablePropertyChanging("SelectedPhoto", _SelectedPhoto, value);

                _SelectedPhoto = value;
                OnPropertyChanged("SelectedPhoto");
            }
        }


        public bool OnlyShowFavorites
        {
            get { return _OnlyShowFavorites; }
            set
            {
                if (value == _OnlyShowFavorites)
                    return;

                // This will make the filter setting change "undoable"
                UndoablePropertyChanging("OnlyShowFavorites", _OnlyShowFavorites, value);

                _OnlyShowFavorites = value;
                OnPropertyChanged("OnlyShowFavorites");
            }
        }

        public string AppTitle
        {
            get
            {
                var name = "";
                var count = "";

                if (null != this.Catalog)
                {
                    name = this.Catalog.Name;
                    count = this.TotalPhotos.ToString();

                    return string.Format("Photo Tagger Demo - Catalog: {0} ({1} Pictures)", name, count);
                }
                else
                {
                    return "Photo Tagger Demo";
                }

            }
        }


        #endregion
        #region Continuous Properties

        /// <summary>
        /// The total number of photos in the catalog. Updates real time.
        /// </summary>
        public int TotalPhotos
        {
            get
            {
                if (null == Catalog)
                    return 0;

                if (null == _TotalPhotos)
                {
                    _TotalPhotos = this.Catalog.Photos.ContinuousCount();
                    _TotalPhotos.PropertyChanged += (s, e) => this.OnPropertyChanged("TotalPhotos");
                }

                return _TotalPhotos.CurrentValue;
            }
        }
        private ContinuousValue<int> _TotalPhotos;
        

        /// <summary>
        /// Returns the total number of favorite photos in the catalog.
        /// Updates real-time with changes to the photo ratings.
        /// </summary>
        public int FavoritePhotos
        {
            get
            {
                if (null == Catalog)
                    return 0;

                // Construct the continuous value on demand.
                if (null == _FavoritePhotos)
                {
                    // Define the logic for the continuous value.
                    _FavoritePhotos = this.Catalog.Photos
                                                  .Where(p => p.IsFavorite)
                                                  .ContinuousCount();
                    
                    // Raise PropertyChanged when this continuous value changes.
                    _FavoritePhotos.PropertyChanged += 
                        (s, e) => this.OnPropertyChanged("FavoritePhotos");
                }

                return _FavoritePhotos.CurrentValue;
            }
        }
        private ContinuousValue<int> _FavoritePhotos;


        #endregion
        #region Commands

        public ICommand PickFolderCommand
        {
            get
            {
                return new GenericCommand(
                    param => PickFolder(),
                    canExec => CanPickFolder()
                );
            }
        }

        public ICommand SaveCommand
        {
            get
            {
                return new GenericCommand(
                    param => Save(),
                    canExec => CanSave()
                );
            }
        }

        public ICommand ExcludePhotoCommand
        {
            get
            {
                return new GenericCommand(
                    param => ExcludePhoto(),
                    canExec => CanExcludePhoto()
                );
            }
        }

        #endregion
        #region Internal

        private bool CanPickFolder()
        {
            return true;
        }

        private void PickFolder()
        {
            var ofd = new OpenFileDialog();
            ofd.DefaultExt = "jpg";
            ofd.Multiselect = true;
            ofd.Title = "Select Photos...";
            ofd.ValidateNames = true;

            var result = ofd.ShowDialog();
            if (result.GetValueOrDefault())
            {
                using (new UndoBatch(Catalog, "Add photos", false))
                {
                    
                    foreach (var file in ofd.FileNames)
                    {
                        var photo = Catalog.Photos.FirstOrDefault(p => p.Path == file);

                        if (null == photo)
                        {
                            using (new DelayedInitializationScope())
                            {
                                photo = new Photo();
                                photo.Title = "<Untitled>";
                                photo.Path = file;
                            }
                        }

                        _Catalog.Photos.Add(photo);
                    }
                }
            }
        } 


        private bool CanSave()
        { 
            return true;
        }

        private void Save()
        {
            var photoRepository = GetPhotoRepository();
            photoRepository.Save(_Catalog);

            // Prevent undo after a save.
            UndoService.Current.Clear();
        }

        
        private bool CanExcludePhoto()
        {
            return null != SelectedPhoto;
        }
        
        private void ExcludePhoto()
        {
            if (CanExcludePhoto())
            {
                using (new UndoBatch(Catalog, "Exclude Photo", false))
                {

                    //NOTE: Removing from the collection does not mean "DELETE".
                    // You must "MarkAsDeleted()" to delete the record.
                    // But don't mark an "Added" entry as deleted or you'll get a concurrency exception.
                    if (SelectedPhoto.ChangeTracker.State != ObjectState.Added)
                        SelectedPhoto.MarkAsDeleted();
                    else
                        Photos.Remove(SelectedPhoto);

                    SelectedPhoto = null;

                }
            }
        }

        private void GetAllPhotos()
        {
            var photoRepository = GetPhotoRepository();
            var catalog = photoRepository.GetCatalog("Main");

            if (null == catalog)
            {
                catalog = new Catalog();
                catalog.Name = "Main";
                photoRepository.Save(catalog);
            }

            Catalog = catalog;

            UndoService.Current.Clear();
        }

        private IPhotoRepository GetPhotoRepository()
        {
            IPhotoRepository photoRepository;

            if (IsInDesignMode)
                photoRepository = new Repository.XML.PhotoRepositoryXML(true);
            else
            {
                if (!_UseSqlRepository.HasValue)
                {
                    var response = MessageBox.Show("This sample can use a SQL database or an XML file for data.\r\n\r\n" +
                                                   "Before using SQL, please create a new database named 'PhotoTaggerDb' and then run the PhotoModel.edmx.sql file to create the tables.\r\n\r\n" +
                                                   "Would you like to use the SQL repository?",
                                                   "Photo Tagger - SQL or XML?", 
                                                   MessageBoxButton.YesNo);

                    if (MessageBoxResult.Yes == response)
                        _UseSqlRepository = true;
                    else
                        _UseSqlRepository = false;
                }

                if (_UseSqlRepository.Value)
                    photoRepository = new Repository.SQL.PhotoRepositorySQL();
                else
                    photoRepository = new Repository.XML.PhotoRepositoryXML();
            }

            return photoRepository;
        }


        #endregion
        #region Undo

        /// <summary>
        /// Call this when a property changes that should be tracked for undo.
        /// </summary>
        /// <param name="propertyName">The name of the property.</param>
        /// <param name="oldValue">The previous value.</param>
        /// <param name="newValue">The new value.</param>
        protected void UndoablePropertyChanging(string propertyName, object oldValue, object newValue)
        {
            DefaultChangeFactory.OnChanging(this, propertyName, oldValue, newValue);
        }

        /// <summary>
        /// Call this when the ViewModel has an ObservableCollection that changed.
        /// The method will attempt to create an undo change item for it.
        /// </summary>
        /// <param name="propertyName">The name of the property that holds the collection.</param>
        /// <param name="collection">The collection instance.</param>
        /// <param name="e">The event args raised by the INotifyCollectionChanged.CollectionChanged event.</param>
        protected void UndoableCollectionChanged(string propertyName, object collection, NotifyCollectionChangedEventArgs e)
        {
            DefaultChangeFactory.OnCollectionChanged(this, propertyName, collection, e);
        }

        public object GetUndoRoot()
        {
            return this.Catalog;
        }

        #endregion
        #region INotifyPropertyChanged

        /// <summary>
        /// The PropertyChanged event is used by consuming code
        /// (like WPF's binding infrastructure) to detect when
        /// a value has changed.
        /// </summary>
        public new event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raise the PropertyChanged event for the 
        /// specified property.
        /// </summary>
        /// <param name="propertyName">
        /// A string representing the name of 
        /// the property that changed.</param>
        /// <remarks>
        /// Only raise the event if the value of the property 
        /// has changed from its previous value</remarks>
        protected void OnPropertyChanged(string propertyName)
        {
            // Validate the property name in debug builds
            VerifyProperty(propertyName);

            if (null != PropertyChanged)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        /// <summary>
        /// Verifies whether the current class provides a property with a given
        /// name. This method is only invoked in debug builds, and results in
        /// a runtime exception if the <see cref="OnPropertyChanged"/> method
        /// is being invoked with an invalid property name. This may happen if
        /// a property's name was changed but not the parameter of the property's
        /// invocation of <see cref="OnPropertyChanged"/>.
        /// </summary>
        /// <param name="propertyName">The name of the changed property.</param>
        [System.Diagnostics.Conditional("DEBUG")]
        private void VerifyProperty(string propertyName)
        {
            Type type = this.GetType();

            // Look for a *public* property with the specified name
            System.Reflection.PropertyInfo pi = type.GetProperty(propertyName);
            if (pi == null)
            {
                // There is no matching property - notify the developer
                string msg = "OnPropertyChanged was invoked with invalid " +
                                "property name {0}. {0} is not a public " +
                                "property of {1}.";
                msg = String.Format(msg, propertyName, type.FullName);
                System.Diagnostics.Debug.Fail(msg);
            }
        }

        #endregion
        #region Design Mode

        // Flag indicating whether the view is loaded in the designer.
        // Access via the property getter.
        private static bool? _isInDesignMode;

        /// <summary>
        /// Gets a value indicating whether the control is in design mode
        /// (running in Blend or Visual Studio).
        /// </summary>
        public static bool IsInDesignMode
        {
            get
            {
                if (!_isInDesignMode.HasValue)
                {
#if SILVERLIGHT
                    _isInDesignMode = DesignerProperties.IsInDesignTool;
#else
                    var prop = DesignerProperties.IsInDesignModeProperty;
                    _isInDesignMode
                        = (bool)DependencyPropertyDescriptor
                        .FromProperty(prop, typeof(FrameworkElement))
                        .Metadata.DefaultValue;

                    // Just to be sure
                    if (!_isInDesignMode.Value
                        && Process.GetCurrentProcess().ProcessName.StartsWith("devenv", StringComparison.Ordinal))
                    {
                        _isInDesignMode = true;
                    }
#endif
                }

                return _isInDesignMode.Value;
            }
        }


        #endregion
    }
}
