﻿using System;
using System.Windows;
using System.Windows.Data;
using System.ComponentModel;
using System.Collections.Generic;

namespace PhotoTagger.Behaviors
{

    /// <summary>
    /// Helper class for enumerating the properties and bindings of a dependency object.
    /// Based on work from Zhou Yong
    /// http://shevaspace.blogspot.com/2007/10/how-to-enumerate-all-bindings-set-on.html
    /// </summary>
    public static class DependencyPropertiesHelper
    {

        public static IEnumerable<DependencyProperty> EnumerateAttachedProperties(DependencyObject element)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }

            //var filter = new Attribute[] { 
            //                new PropertyFilterAttribute(
            //                    PropertyFilterOptions.SetValues |  
            //                    PropertyFilterOptions.UnsetValues | 
            //                    PropertyFilterOptions.Valid
            //                ) 
            //             };

            var filter = new Attribute[] { 
                            new PropertyFilterAttribute(PropertyFilterOptions.All) 
                         };

            foreach (PropertyDescriptor pd in TypeDescriptor.GetProperties(element, filter))
            {
                DependencyPropertyDescriptor dpd = DependencyPropertyDescriptor.FromProperty(pd);
                if (dpd != null && dpd.IsAttached)
                    yield return dpd.DependencyProperty;
            }
        }

        public static IEnumerable<DependencyProperty> EnumerateProperties(DependencyObject element)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }

            //var filter = new Attribute[] { 
            //                new PropertyFilterAttribute(
            //                    PropertyFilterOptions.SetValues |  
            //                    PropertyFilterOptions.UnsetValues | 
            //                    PropertyFilterOptions.Valid
            //                ) 
            //             };

            var filter = new Attribute[] { 
                            new PropertyFilterAttribute(PropertyFilterOptions.All) 
                         };

            foreach (PropertyDescriptor pd in TypeDescriptor.GetProperties(element, filter))
            {
                DependencyPropertyDescriptor dpd = DependencyPropertyDescriptor.FromProperty(pd);

                if (dpd != null)
                    yield return dpd.DependencyProperty;
            }
        }

        /// <summary>
        /// This version will iterate over values that are set "locally".
        /// NOTE: elements that are part of a DataTemplate are not "Local".
        /// </summary>
        public static IEnumerable<Tuple<DependencyProperty, Binding>> EnumerateLocalBindings(DependencyObject element)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }

            LocalValueEnumerator lve = element.GetLocalValueEnumerator();

            while (lve.MoveNext())
            {
                LocalValueEntry entry = lve.Current;

                if (BindingOperations.IsDataBound(element, entry.Property))
                {
                    Binding binding = (entry.Value as BindingExpression).ParentBinding;
                    yield return new Tuple<DependencyProperty, Binding>(entry.Property, binding);
                }
            }
        }


        public static IEnumerable<Tuple<DependencyProperty, Binding>> EnumerateAllBindings(DependencyObject element)
        {
            if (element == null)
            {
                throw new ArgumentNullException("element");
            }

            //var filter = new Attribute[] { 
            //                new PropertyFilterAttribute(
            //                        PropertyFilterOptions.SetValues | 
            //                        // PropertyFilterOptions.UnsetValues | 
            //                        PropertyFilterOptions.Valid) 
            //             };

            var filter = new Attribute[] { 
                            new PropertyFilterAttribute(PropertyFilterOptions.All) 
                         };

            var propDescriptions = TypeDescriptor.GetProperties(element, filter);

            foreach (PropertyDescriptor pd in propDescriptions)
            {
                DependencyPropertyDescriptor dpd = DependencyPropertyDescriptor.FromProperty(pd);
                if (null != dpd && BindingOperations.IsDataBound(element, dpd.DependencyProperty))
                {
                    var binding = BindingOperations.GetBinding(element, dpd.DependencyProperty);
                    yield return new Tuple<DependencyProperty, Binding>(dpd.DependencyProperty, binding);
                }
            }
        }


        public static IEnumerable<Tuple<DependencyProperty, BindingExpression>> EnumerateAllBindingExpressions(DependencyObject element)
        {
            if (element == null)
                throw new ArgumentNullException("element");

            var fe = element as FrameworkElement;
            var fce = element as FrameworkContentElement;

            if (null == fe && null == fce)
                throw new ArgumentException("Element must be a FrameworkElement or a FrameworkContentElement.");

            foreach (var binding in EnumerateAllBindings(element))
            {
                if (binding != null)
                {
                    BindingExpression be;

                    if (null != fe)
                        be = fe.GetBindingExpression(binding.Item1);
                    else
                        be = fce.GetBindingExpression(binding.Item1);

                    yield return new Tuple<DependencyProperty, BindingExpression>(binding.Item1, be);
                }

            }

        }

    }
}
