﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows;
using ViewModelLocatorsWithConverters.Model;
using ViewModelLocatorsWithConverters.ViewModels;

namespace ViewModelLocatorsWithConverters.Locators
{

    /// <summary>
    /// Stores a cache of view model instances. Also detects design time mode.
    /// </summary>
    /// <remarks>
    /// This is an internal class for use by the ViewModelLocator classes.
    /// Portions of this implementation are based on work by Laurent Bugnion (Galasoft).
    /// </remarks>
    public static class ViewModelLocator
    {

        #region Member Variables

        // Store a list of ViewModel instances, keyed by their associated model instance.
        // Key is a composite of the related instance plus the type of viewmodel. 
        // This is because a given instance could have multiple views.
        private static Dictionary<Tuple<object, Type>, ViewModelBase> _ViewModelCache 
            = new Dictionary<Tuple<object, Type>, ViewModelBase>();

        // Flag indicating whether the view is loaded in the designer.
        // Access via the property getter.
        private static bool? _isInDesignMode;

        #endregion

        #region Constructors

        #endregion

        #region Public Methods


        /// <summary>
        /// Gets a value indicating whether the control is in design mode
        /// (running in Blend or Visual Studio).
        /// </summary>
        public static bool IsInDesignMode
        {
            get
            {
                if (!_isInDesignMode.HasValue)
                {
#if SILVERLIGHT
                    _isInDesignMode = DesignerProperties.IsInDesignTool;
#else
                    var prop = DesignerProperties.IsInDesignModeProperty;
                    _isInDesignMode
                        = (bool)DependencyPropertyDescriptor
                        .FromProperty(prop, typeof(FrameworkElement))
                        .Metadata.DefaultValue;

                    // Just to be sure
                    if (!_isInDesignMode.Value
                        && Process.GetCurrentProcess().ProcessName.StartsWith("devenv", StringComparison.Ordinal))
                    {
                        _isInDesignMode = true;
                    }
#endif
                }

                return _isInDesignMode.Value;
            }
        }


        #endregion

        #region Internal Methods

        /// <summary>
        /// Get a previously cached ViewModel from the cache.
        /// </summary>
        /// <typeparam name="T">The type of the ViewModel instance.</typeparam>
        /// <param name="model">
        /// A reference to the instance of the model that is related to this ViewModel instance.
        /// This can be null if there is no model, as is often the case for the "main" or "root" ViewModel.
        /// </param>
        /// <returns>The cached ViewModel, or null if none is found.</returns>
        internal static T GetViewModel<T>(object model) where T : ViewModelBase
        {
            var key = new Tuple<object, Type>(model, typeof(T));
            if (_ViewModelCache.ContainsKey(key))
                return _ViewModelCache[key] as T;
            else
                return null;
        }

        /// <summary>
        /// Cache a ViewModel so that it can be used again if/when the domain model is
        /// displayed again.
        /// </summary>
        /// <param name="model">
        /// A reference to the instance of the model that is related to this ViewModel instance.
        /// This can be null if there is no model, as is often the case for the "main" or "root" ViewModel.
        /// </param>
        /// <param name="viewModel">
        /// The instance of the ViewModel to cache.
        /// </param>
        internal static void CacheViewModel(object model, ViewModelBase viewModel)
        {
            var key = new Tuple<object, Type>(model, viewModel.GetType());
            _ViewModelCache.Add(key, viewModel);
        }

        /// <summary>
        /// Remove a ViewModel from the cache.
        /// </summary>
        /// <typeparam name="T">The type of the ViewModel to clear.</typeparam>
        /// <param name="model">The instance of the model that this ViewModel was linked to. This can be null.</param>
        internal static void ClearViewModel<T>(object model) where T : ViewModelBase
        {
            var key = new Tuple<object, Type>(model, typeof(T));
            ViewModelBase vm = null;

            if (_ViewModelCache.ContainsKey(key))
                vm = _ViewModelCache[key];

            if (vm != null)
            {
                _ViewModelCache.Remove(key);
            }
        }

        #endregion

        #region Designer Data Helpers

        /// <summary>
        /// Create some instances of the domain model for use during design time.
        /// </summary>
        /// <returns></returns>
        internal static Family InitDesignTimeInstance()
        {
            Family family = new Family() { 
                FamilyName = "Design Time Family" 
            };

            family.Members.Add(new Person()
            {
                Name = "John Doe",
                Role = "Father"
            });

            family.Members.Add(new Person()
            {
                Name = "Jane Doe",
                Role = "Mother"
            });

            family.Members.Add(new Person()
            {
                Name = "Jimmy Doe",
                Role = "Child"
            });

            family.Members.Add(new Person()
            {
                Name = "Jill Doe",
                Role = "Child"
            });

            family.Members.Add(new Person()
            {
                Name = "Spot",
                Role = "Dog"
            });
            
            return family;
        }

        #endregion

    }
}
